#!/bin/bash

export LANGUAGE=en_US.UTF-8
export LANG=en_US.UTF-8
export LC_ALL=en_US.UTF-8

LOGFILE="/var/log/network_settings.log"
exec 2>>$LOGFILE

if [ -f /proc/user_beancounters ]; then
  echo "This is an OpenVZ Container. Network settings may not be changed from inside the VPS."
  exit
fi

if [ -f /dev/incus/sock ]; then
  echo "This is an Incus Container. Network settings may not be changed from inside the VPS."
  exit
fi

: ${DIALOG=dialog}

# Get a list of network interfaces excluding loopback and virtual interfaces
INTERFACE=$(ip -o link show | awk -F': ' '{print $2}' | grep -E -v '^(lo|vir|incus|br|veth|docker)'|head -1|cut -d \@ -f1)

# Check if any interface was found
if [ -n "$INTERFACE" ]; then
  REAL_PRIMARY_INTERFACE_NAME=$INTERFACE
else
  REAL_PRIMARY_INTERFACE_NAME="eth0"
fi

function waitForCceConstruct() {
  local waitFor="cce_construct"
  while pgrep -f $waitFor > /dev/null; do
    sleep 1
    echo "75" | ${DIALOG} --nocancel --backtitle "${CDTITLE}" --title "${TITLE}" --gauge "BlueOnyx 5211R initial setup:\nPlease wait for CCEd Constructors to finish ..." 10 70 0
    sleep 3
  done
  echo "100" | ${DIALOG} --nocancel --backtitle "${CDTITLE}" --title "${TITLE}" --gauge "BlueOnyx 5211R initial setup:\nPlease wait for CCEd Constructors to finish ..." 10 70 0
  sleep 3
}

function detectActiveInterface() {
  if ip link show br0 &>/dev/null && ip addr show br0 | grep 'inet ' &>/dev/null; then
    ACTIVE_IFACE="br0"
  else
    ACTIVE_IFACE=$(ip route | grep default | awk '{print $5}')
  fi
}

function cidr_to_quad() {
  local cidr=$1
  local mask=""
  local a=$((0xffffffff << (32 - cidr))); 
  for i in {1..4}; do 
    mask+=$(( (a >> ((4-i) * 8)) & 0xff ))
    [ $i -lt 4 ] && mask+=.
  done
  echo $mask
}

function readNetworkSettings() {
  local iface=$1
  SERVERNAME=$(uname -n | head -1)
  IPADDRESS=$(ip -4 addr show ${iface} | grep -v '127.0.0.1' | grep -v ::1 | grep -oP '(?<=inet\s)\d+(\.\d+){3}' | head -1)
  NETMASK=$(ip -4 addr show ${iface} | grep -oP '(?<=inet\s)\d+(\.\d+){3}/\d+' | cut -d'/' -f2)
  NETMASK=$(cidr_to_quad $NETMASK)
  DEFAULTGW=$(ip route | awk '/default/ {print $3}' | head -1)
  DNSSERVER=$(grep ^nameserver /etc/resolv.conf | awk '{print $2}' | head -1)
  IPV6ADDRESS=$(ip -6 addr show ${iface} | grep -v fe80 | grep -oP '(?<=inet6\s)[^/]+' | head -1)
  DEFAULTGWV6=$(LC_ALL=C ip -6 route show dev ${iface} | awk '/default/ {print $3}'|head -1)
}

function update_firewalld_rules() {
  local old_iface=$1
  local new_iface=$2

  # Detect the zone of the old interface
  ZONE=$(firewall-cmd --get-zone-of-interface=${old_iface})
  
  if [ -n "$ZONE" ]; then
    # Remove the old interface from the zone
    firewall-cmd --zone=${ZONE} --remove-interface=${old_iface} --permanent &>/dev/null || :

    # Add the new interface to the same zone
    firewall-cmd --zone=${ZONE} --add-interface=${new_iface} --permanent &>/dev/null || :

    # Reload firewalld to apply changes
    firewall-cmd --reload &>/dev/null || :
  fi
}

waitForCceConstruct
detectActiveInterface
readNetworkSettings ${ACTIVE_IFACE}

function validate_ip() {
  local ip=$1
  if [[ ${ip} =~ ^([0-9]{1,3}\.){3}[0-9]{1,3}$ ]]; then
    return 0
  else
    return 1
  fi
}

function getUserInput() {
  local msg="$1"
  local value="$2"
  local input=$(${DIALOG} --nocancel --backtitle "${CDTITLE}" --title "${TITLE}" --inputbox "${msg}" 15 70 "${value}" 2>&1 >/dev/tty)
  echo "${input}"
}

function askNetworkType() {
  ${DIALOG} --backtitle "${CDTITLE}" --title "${TITLE}" --menu "Please choose the network configuration:" 15 70 3 \
    1 "Traditional eth0 primary interface (default)" \
    2 "Bridged br0 network (for Incus Containers)" \
    3 "DHCP" 2>&1 >/dev/tty
}

function askYesNo() {
  local msg="$1"
  ${DIALOG} --backtitle "${CDTITLE}" --title "${TITLE}" --yesno "${msg}" 10 60
}

function configureDHCP() {
  #SERVERNAME=$(getUserInput "Please Enter the Fully Qualified Domain Name of your Server (FQDN)" "${SERVERNAME}")

  # Turn off bridged mode:
  /usr/sausalito/sbin/set_system_bridge.pl --bridged_network=0

  if [ "$SERVERNAME" != "" ];then
    # And we're really going to town with this:
    /usr/sausalito/sbin/set_servername.pl configure $SERVERNAME >/dev/null 2>&1 || :
  fi

  # Turn on DHCP:
  /usr/sausalito/sbin/dhcp_enable.pl --confirm=yes --enable >/dev/null 2>&1 || :

  echo "20" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0

  sleep 3

  echo "30" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0

  # Run /usr/sausalito/sbin/network_apply_settings.pl to apply the network settings:
  /usr/sausalito/sbin/network_apply_settings.pl &>/dev/null || :

  # Fetch the primary interface name again as it might have changed from bridge back to the real device:
  INTERFACE=$(ip -o link show | awk -F': ' '{print $2}' | grep -E -v '^(lo|vir|docker)')

  # Check if any interface was found
  if [ -n "$INTERFACE" ]; then
    REAL_PRIMARY_INTERFACE_NAME=$INTERFACE
  else
    REAL_PRIMARY_INTERFACE_NAME="eth0"
  fi

  echo "40" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0

  # Restart services
  systemctl restart rsyslog &>/dev/null || :
  systemctl enable NetworkManager.service &>/dev/null || :

  echo "50" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0

  # Enable swatch:
  systemctl enable swatch.service &>/dev/null || :

  # Restart daemons
  systemctl restart httpd &>/dev/null || :
  echo "60" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0
  systemctl restart admserv &>/dev/null || :

  echo "70" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0

  echo "80" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0
  systemctl restart firewalld &>/dev/null || :

  # Rehash CCEd
  echo "90" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0
  /usr/sausalito/sbin/cced.init rehash &>/dev/null || :

  echo "100" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0

  # Run cce_construct_dialog
  /usr/sausalito/sbin/cce_construct_dialog
  
  # Run Swatch
  /usr/sausalito/sbin/swatch.sh &>/dev/null || :

  # Restart NetworkManager to apply the changes
  systemctl restart NetworkManager

  # Runtime issue: eth0/br0 needs a moment:
  sleep 5

  #Update /etc/issue:
  {
    echo "BlueOnyx 5211R on \S"
    echo "Kernel \r on an \m"
    echo ""
  } > /etc/issue

  # Clean up /root/.bashrc
  grep -v "network_settings.sh" /root/.bashrc > /tmp/.bashrc && mv /tmp/.bashrc /root/.bashrc

  # Update /etc/sysconfig/network:
  {
    echo "HOSTNAME=${SERVERNAME}"
    echo "NETWORKING=yes"
    echo "NOZEROCONF=yes"
  } > /etc/sysconfig/network

  #systemctl restart NetworkManager
  systemctl restart httpd &>/dev/null || :
  systemctl restart admserv &>/dev/null || :

  {
    echo ""
    echo "DHCP Network Setup of BlueOnyx 5211R has been completed."
    echo "========================================================="
    echo ""
    echo "You can reach the GUI at these two URL's in your browser:"
    echo ""
    echo "http://$(ip -4 addr show ${REAL_PRIMARY_INTERFACE_NAME} | grep -oP '(?<=inet\s)\d+(\.\d+){3}' | head -1):444/login"
    echo ""
    echo "https://$(ip -4 addr show ${REAL_PRIMARY_INTERFACE_NAME} | grep -oP '(?<=inet\s)\d+(\.\d+){3}' | head -1):81/login"
    echo ""
    echo "Enjoy BlueOnyx 5211R!"
    echo ""
  } > /tmp/.bxgreeting
  ${DIALOG} --nocancel --backtitle "${CDTITLE}" --title "${TITLE}" --textbox /tmp/.bxgreeting 22 70
  rm -f /tmp/.bxgreeting
}

SERVERNAME=$(getUserInput "Please Enter the Fully Qualified Domain Name of your Server (FQDN)" "${SERVERNAME}")
NETWORK_TYPE=$(askNetworkType)
DIALOG_EXIT_STATUS=$?

if [ ${DIALOG_EXIT_STATUS} -eq 1 ]; then
  ${DIALOG} --msgbox "No changes made. Exiting..." 10 40
  exit 1
fi

if [ "${NETWORK_TYPE}" -eq 1 ]; then
  USE_BRIDGE=0
  ENABLE_DHCP=0
elif [ "${NETWORK_TYPE}" -eq 2 ]; then
  USE_BRIDGE=1
  ENABLE_DHCP=0
else
  ENABLE_DHCP=1
  configureDHCP
  exit 0
fi

askYesNo "Do you want to configure IPv4?"
if [ $? -eq 0 ]; then
  CONFIGURE_IPV4=1
  IPADDRESS=$(getUserInput "Please Enter your IPv4 IP Address" "${IPADDRESS}")
  NETMASK=$(getUserInput "Please Enter your Netmask" "${NETMASK}")
  DEFAULTGW=$(getUserInput "Please Enter your Default IPv4 Gateway" "${DEFAULTGW}")
  DNSSERVER=$(getUserInput "Please Enter your DNS Server IP" "${DNSSERVER}")
  if ! validate_ip "${IPADDRESS}"; then
    echo "Invalid IPv4 address format"
    exit 1
  fi
  if ! validate_ip "${DEFAULTGW}"; then
    echo "Invalid IPv4 gateway format"
    exit 1
  fi
else
  CONFIGURE_IPV4=0
fi

askYesNo "Do you want to configure IPv6?"
if [ $? -eq 0 ]; then
  CONFIGURE_IPV6=1
  IPV6ADDRESS=$(getUserInput "Please Enter your IPv6 IP Address" "${IPV6ADDRESS}")
  DEFAULTGWV6=$(getUserInput "Please Enter your Default IPv6 Gateway" "${DEFAULTGWV6}")
else
  CONFIGURE_IPV6=0
fi

function get_first_uuid_for_device() {
  local device_name=$1
  local uuid=$(/usr/sausalito/sbin/get_nmcli_uuid.pl --device=${device_name} | head -1)
  echo ${uuid#UUID:}
}

function configureNetwork() {
  echo percentage | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "text" height width percent
  echo "10" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0

  # Change the default gateway
  /usr/bin/perl -pi -e "s#GATEWAY=.*#GATEWAY=${DEFAULTGW}#" /etc/sysconfig/network

  # Find the Network ID & Broadcast ID
  NETWORKID=$(/usr/sausalito/sbin/minicalc.pl id "${IPADDRESS}" "${NETMASK}")
  BROADCAST=$(/usr/sausalito/sbin/minicalc.pl bid "${IPADDRESS}" "${NETMASK}")

  # Update firewalld rules
  if [ ${USE_BRIDGE} -eq 1 ]; then
    update_firewalld_rules "${REAL_PRIMARY_INTERFACE_NAME}" "br0"
  else
    update_firewalld_rules "br0" "${REAL_PRIMARY_INTERFACE_NAME}"
  fi

  # Configure /etc/sysconfig/network
  {
    echo "FORWARD_IPV4=false"
    if [ "${DEFAULTGW}" != "" ]; then
      echo "GATEWAY=${DEFAULTGW}"
      echo "NETWORKING=yes"
    fi
    echo "HOSTNAME=${SERVERNAME}"
    echo "IPV6_AUTOCONF=no"
    if [ "${DEFAULTGWV6}" != "" ]; then
      echo "IPV6FORWARDING=yes"
      echo "IPV6_DEFAULTDEV=${USE_BRIDGE:+br0}"
      echo "IPV6_DEFAULTGW=${DEFAULTGWV6}"
      sleep 5
    fi
    echo "NOZEROCONF=yes"
  } > /etc/sysconfig/network

  # Restart services
  systemctl restart rsyslog &>/dev/null || :
  systemctl enable NetworkManager.service &>/dev/null || :
  systemctl enable swatch.service &>/dev/null || :

  # Set the bridged_network flag in the System object
  if [ ${USE_BRIDGE} -eq 1 ]; then
    /usr/sausalito/sbin/set_system_bridge.pl --bridged_network=1
  else
    /usr/sausalito/sbin/set_system_bridge.pl --bridged_network=0
  fi

  if [ "$SERVERNAME" != "" ];then
    # And we're really going to town with this:
    /usr/sausalito/sbin/set_servername.pl configure $SERVERNAME >/dev/null 2>&1 || :
  fi

  echo "20" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0

  # Turn off DHCP:
  /usr/sausalito/sbin/dhcp_enable.pl --confirm=yes --disable >/dev/null 2>&1 || :

  echo "30" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0

  # Additional configuration through nethelper.pl
  if [ "${CONFIGURE_IPV4}" -eq 1 ] && [ "${CONFIGURE_IPV6}" -eq 1 ]; then
    /usr/sausalito/sbin/nethelper.pl --device=${REAL_PRIMARY_INTERFACE_NAME} --ipv4ip="${IPADDRESS}" --ipv4nm="${NETMASK}" --ipv4gw="${DEFAULTGW}" --ipv6="${IPV6ADDRESS}" --ipv6gw="${DEFAULTGWV6}" --dns="${DNSSERVER}"
  elif [ "${CONFIGURE_IPV4}" -eq 1 ]; then
    /usr/sausalito/sbin/nethelper.pl --device=${REAL_PRIMARY_INTERFACE_NAME} --ipv4ip="${IPADDRESS}" --ipv4nm="${NETMASK}" --ipv4gw="${DEFAULTGW}" --dns="${DNSSERVER}"
  elif [ "${CONFIGURE_IPV6}" -eq 1 ]; then
    /usr/sausalito/sbin/nethelper.pl --device=${REAL_PRIMARY_INTERFACE_NAME} --ipv6="${IPV6ADDRESS}" --ipv6gw="${DEFAULTGWV6}" --dns="${DNSSERVER}"
  fi

  # Run /usr/sausalito/sbin/network_apply_settings.pl to apply the network settings:
  /usr/sausalito/sbin/network_apply_settings.pl &>/dev/null || :

  echo "40" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0

  echo "50" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0

  # Restart daemons
  systemctl restart httpd &>/dev/null || :
  echo "60" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0
  systemctl restart admserv &>/dev/null || :

  echo "70" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0

  echo "80" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0
  systemctl restart firewalld &>/dev/null || :

  # Rehash CCEd
  echo "90" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0
  /usr/sausalito/sbin/cced.init rehash &>/dev/null || :

  echo "100" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0

  # Run cce_construct_dialog
  /usr/sausalito/sbin/cce_construct_dialog
  
  # Run Swatch
  /usr/sausalito/sbin/swatch.sh &>/dev/null || :

  # Restart NetworkManager to apply the changes
  systemctl restart NetworkManager

  # Runtime issue: eth0/br0 needs a moment:
  sleep 5

  {
    echo "BlueOnyx 5211R on \S"
    echo "Kernel \r on an \m"
    echo ""
    if [ "${IPADDRESS}" != "" ]; then
      echo "IPv4 Address: ${IPADDRESS}"
    fi
    if [ "${IPV6ADDRESS}" != "" ]; then
      echo "IPv6 Address: ${IPV6ADDRESS}"
    fi
    echo ""
  } > /etc/issue

  # Clean up /root/.bashrc
  grep -v "network_settings.sh" /root/.bashrc > /tmp/.bashrc && mv /tmp/.bashrc /root/.bashrc

  # Display end message
  {
    echo ""
    echo "Network Setup of BlueOnyx 5211R has been completed."
    echo "========================================================="
    echo ""
    echo "You can reach the GUI at these two URL's in your browser:"
    echo ""
    echo "http://${IPADDRESS}:444/login"
    echo ""
    echo "https://${IPADDRESS}:81/login"
    echo ""
    echo "Enjoy BlueOnyx 5211R!"
    echo ""
  } > /tmp/.bxgreeting
  ${DIALOG} --nocancel --backtitle "${CDTITLE}" --title "${TITLE}" --textbox /tmp/.bxgreeting 22 70
  rm -f /tmp/.bxgreeting

  # Remove old br0 configuration after confirming eth0 is up
  if [ ${USE_BRIDGE} -eq 0 ]; then
    ifdown br0 &>/dev/null || :
    rm -f /etc/sysconfig/network-scripts/ifcfg-br0
  fi
}

configureNetwork

# 
# Copyright (c) 2019-2024 Michael Stauber, SOLARSPEED.NET
# Copyright (c) 2019-2024 Team BlueOnyx, BLUEONYX.IT
# All Rights Reserved.
# 
# 1. Redistributions of source code must retain the above copyright 
#   notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright 
#   notice, this list of conditions and the following disclaimer in 
#   the documentation and/or other materials provided with the 
#   distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its 
#   contributors may be used to endorse or promote products derived 
#   from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.
# 
